@@ -35,8 +35,11 @@
 #include <gio/gio.h>
 #include <libxml/tree.h>
 
-#include "na-action-profile.h"
-#include "na-gconf-keys.h"
+#include <common/na-iprefs.h>
+#include <common/na-object-api.h>
+
+#include <runtime/na-gconf-provider-keys.h>
+
 #include "na-utils.h"
 #include "na-xml-names.h"
 #include "na-xml-writer.h"
@@ -57,10 +60,10 @@ struct NAXMLWriterPrivate {
 /* instance properties
  */
 enum {
-	PROP_GCONF_WRITER_UUID = 1
+	XML_WRITER_PROP_UUID_ID = 1
 };
 
-#define PROP_GCONF_WRITER_UUID_STR		"gconf-writer-uuid"
+#define XLM_WRITER_PROP_UUID		"na-xml-writer-uuid"
 
 static GObjectClass *st_parent_class = NULL;
 
@@ -73,7 +76,7 @@ static void         instance_dispose( GObject *object );
 static void         instance_finalize( GObject *object );
 
 static NAXMLWriter *xml_writer_new( const gchar *uuid );
-static xmlDocPtr    create_xml_schema( NAXMLWriter *writer, gint format, const NAAction *action );
+static xmlDocPtr    create_xml_schema( NAXMLWriter *writer, gint format, const NAObjectAction *action );
 static void         create_schema_entry(
 								NAXMLWriter *writer,
 								gint format,
@@ -86,7 +89,7 @@ static void         create_schema_entry(
 								gboolean is_l10n_value,
 								const gchar *short_desc,
 								const gchar *long_desc );
-static xmlDocPtr    create_xml_dump( NAXMLWriter *writer, gint format, const NAAction *action );
+static xmlDocPtr    create_xml_dump( NAXMLWriter *writer, gint format, const NAObjectAction *action );
 static void         create_dump_entry(
 								NAXMLWriter *writer,
 								gint format,
@@ -163,11 +166,11 @@ class_init( NAXMLWriterClass *klass )
 	object_class->set_property = instance_set_property;
 
 	spec = g_param_spec_string(
-			PROP_GCONF_WRITER_UUID_STR,
-			PROP_GCONF_WRITER_UUID_STR,
-			"UUID of the action", "",
+			XLM_WRITER_PROP_UUID,
+			XLM_WRITER_PROP_UUID,
+			"UUID of the action for which we invoke this XMLWriter", "",
 			G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE );
-	g_object_class_install_property( object_class, PROP_GCONF_WRITER_UUID, spec );
+	g_object_class_install_property( object_class, XML_WRITER_PROP_UUID_ID, spec );
 
 	klass->private = g_new0( NAXMLWriterClassPrivate, 1 );
 }
@@ -179,7 +182,7 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	NAXMLWriter *self;
 
 	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
-	g_assert( NA_IS_XML_WRITER( instance ));
+	g_return_if_fail( NA_IS_XML_WRITER( instance ));
 	self = NA_XML_WRITER( instance );
 
 	self->private = g_new0( NAXMLWriterPrivate, 1 );
@@ -192,17 +195,20 @@ instance_get_property( GObject *object, guint property_id, GValue *value, GParam
 {
 	NAXMLWriter *self;
 
-	g_assert( NA_IS_XML_WRITER( object ));
+	g_return_if_fail( NA_IS_XML_WRITER( object ));
 	self = NA_XML_WRITER( object );
 
-	switch( property_id ){
-		case PROP_GCONF_WRITER_UUID:
-			g_value_set_string( value, self->private->uuid );
-			break;
+	if( !self->private->dispose_has_run ){
 
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
-			break;
+		switch( property_id ){
+			case XML_WRITER_PROP_UUID_ID:
+				g_value_set_string( value, self->private->uuid );
+				break;
+
+			default:
+				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
+				break;
+		}
 	}
 }
 
@@ -211,27 +217,32 @@ instance_set_property( GObject *object, guint property_id, const GValue *value,
 {
 	NAXMLWriter *self;
 
-	g_assert( NA_IS_XML_WRITER( object ));
+	g_return_if_fail( NA_IS_XML_WRITER( object ));
 	self = NA_XML_WRITER( object );
 
-	switch( property_id ){
-		case PROP_GCONF_WRITER_UUID:
-			g_free( self->private->uuid );
-			self->private->uuid = g_value_dup_string( value );
-			break;
+	if( !self->private->dispose_has_run ){
 
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
-			break;
+		switch( property_id ){
+			case XML_WRITER_PROP_UUID_ID:
+				g_free( self->private->uuid );
+				self->private->uuid = g_value_dup_string( value );
+				break;
+
+			default:
+				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
+				break;
+		}
 	}
 }
 
 static void
 instance_dispose( GObject *object )
 {
+	static const gchar *thisfn = "na_xml_writer_instance_dispose";
 	NAXMLWriter *self;
 
-	g_assert( NA_IS_XML_WRITER( object ));
+	g_debug( "%s: object=%p", thisfn, ( void * ) object );
+	g_return_if_fail( NA_IS_XML_WRITER( object ));
 	self = NA_XML_WRITER( object );
 
 	if( !self->private->dispose_has_run ){
@@ -239,7 +250,9 @@ instance_dispose( GObject *object )
 		self->private->dispose_has_run = TRUE;
 
 		/* chain up to the parent class */
-		G_OBJECT_CLASS( st_parent_class )->dispose( object );
+		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
+			G_OBJECT_CLASS( st_parent_class )->dispose( object );
+		}
 	}
 }
 
@@ -248,7 +261,7 @@ instance_finalize( GObject *object )
 {
 	NAXMLWriter *self;
 
-	g_assert( NA_IS_XML_WRITER( object ));
+	g_return_if_fail( NA_IS_XML_WRITER( object ));
 	self = NA_XML_WRITER( object );
 
 	g_free( self->private->uuid );
@@ -256,7 +269,7 @@ instance_finalize( GObject *object )
 	g_free( self->private );
 
 	/* chain call to parent class */
-	if( st_parent_class->finalize ){
+	if( G_OBJECT_CLASS( st_parent_class )->finalize ){
 		G_OBJECT_CLASS( st_parent_class )->finalize( object );
 	}
 }
@@ -264,59 +277,71 @@ instance_finalize( GObject *object )
 static NAXMLWriter *
 xml_writer_new( const gchar *uuid )
 {
-	return( g_object_new( NA_XML_WRITER_TYPE, PROP_GCONF_WRITER_UUID_STR, uuid, NULL ));
+	return( g_object_new( NA_XML_WRITER_TYPE, XLM_WRITER_PROP_UUID, uuid, NULL ));
 }
 
 /**
  * na_xml_writer_export:
- * @action: the #NAAction to be exported.
+ * @action: the #NAObjectAction to be exported.
+ * Can be NULL when exporting schemas ; in this case, format must be
+ * FORMAT_GCONF_SCHEMA.
  * @folder: the directoy where to write the output XML file.
- *  If NULL, the output will be directed to stdout.
- *  If not NULL, must be specified as an URI (e.g. file:///tmp)
+ * If NULL, the output will be directed to stdout.
  * @format: the export format.
- * @msg: pointer to a buffer which will receive error message.
+ * @msg: pointer to a buffer which will receive error messages.
  *
  * Export the specified action as an XML file.
  *
- * Returns: the written filename, or NULL if written to stdout or error.
+ * Returns: the written filename, or NULL if written to stdout.
  */
 gchar *
-na_xml_writer_export( const NAAction *action, const gchar *folder, gint format, gchar **msg )
+na_xml_writer_export( const NAObjectAction *action, const gchar *folder, gint format, GSList **msg )
 {
+	static const gchar *thisfn = "na_xml_writer_export";
 	gchar *filename = NULL;
 	gchar *xml_buffer;
 
+	g_debug( "%s: action=%p, format=%u", thisfn, ( void * ) action, format );
+	g_return_val_if_fail( action || format == IPREFS_EXPORT_FORMAT_GCONF_SCHEMA, NULL );
+
 	switch( format ){
-		case FORMAT_GCONFSCHEMAFILE_V1:
-		case FORMAT_GCONFSCHEMAFILE_V2:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2:
 			filename = na_xml_writer_get_output_fname( action, folder, format );
 			break;
 
 		/* this is the format used by nautilus-actions-new utility,
 		 * and that's why this option takes care of a NULL folder
 		 */
-		case FORMAT_GCONFENTRY:
+		case IPREFS_EXPORT_FORMAT_GCONF_ENTRY:
 			if( folder ){
 				filename = na_xml_writer_get_output_fname( action, folder, format );
 			}
 			break;
 
-		/* this is the format used by nautilus-actions-install-schema
+		/* this is the format used by nautilus-actions-schemas
 		 * utility, and that's why this option takes care of a NULL
 		 * folder, or an output filename
 		 */
-		case FORMAT_GCONFSCHEMA:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA:
 			if( folder ){
 				filename = g_strdup( folder );
 			}
 			break;
+
+		default:
+			g_return_val_if_reached( NULL );
 	}
 
-	g_return_val_if_fail( !folder || filename, NULL );
+	g_assert( filename || folder == NULL );
 
 	xml_buffer = na_xml_writer_get_xml_buffer( action, format );
 
-	na_xml_writer_output_xml( xml_buffer, filename );
+	if( folder ){
+		na_xml_writer_output_xml( xml_buffer, filename );
+	} else {
+		g_print( "%s", xml_buffer );
+	}
 
 	g_free( xml_buffer );
 
@@ -325,7 +350,7 @@ na_xml_writer_export( const NAAction *action, const gchar *folder, gint format,
 
 /**
  * na_xml_writer_get_output_fname:
- * @action: the #NAAction to be exported.
+ * @action: the #NAObjectAction to be exported.
  * @folder: the uri of the directoy where to write the output XML file.
  * @format: the export format.
  *
@@ -341,7 +366,7 @@ na_xml_writer_export( const NAAction *action, const gchar *folder, gint format,
  * between our test of inexistance and the actual write.
  */
 gchar *
-na_xml_writer_get_output_fname( const NAAction *action, const gchar *folder, gint format )
+na_xml_writer_get_output_fname( const NAObjectAction *action, const gchar *folder, gint format )
 {
 	gchar *uuid;
 	gchar *canonical_fname = NULL;
@@ -353,20 +378,20 @@ na_xml_writer_get_output_fname( const NAAction *action, const gchar *folder, gin
 	g_return_val_if_fail( folder, NULL );
 	g_return_val_if_fail( strlen( folder ), NULL );
 
-	uuid = na_action_get_uuid( action );
+	uuid = na_object_get_id( action );
 
 	switch( format ){
-		case FORMAT_GCONFSCHEMAFILE_V1:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1:
 			canonical_fname = g_strdup_printf( "config_%s", uuid );
 			canonical_ext = g_strdup( "schemas" );
 			break;
 
-		case FORMAT_GCONFSCHEMAFILE_V2:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2:
 			canonical_fname = g_strdup_printf( "config-%s", uuid );
 			canonical_ext = g_strdup( "schema" );
 			break;
 
-		case FORMAT_GCONFENTRY:
+		case IPREFS_EXPORT_FORMAT_GCONF_ENTRY:
 			canonical_fname = g_strdup_printf( "action-%s", uuid );
 			canonical_ext = g_strdup( "xml" );
 			break;
@@ -399,7 +424,7 @@ na_xml_writer_get_output_fname( const NAAction *action, const gchar *folder, gin
 
 /**
  * na_xml_writer_get_xml_buffer:
- * @action: the #NAAction to be exported.
+ * @action: the #NAObjectAction to be exported.
  * @format: the export format.
  *
  * Returns: a buffer which contains the XML output.
@@ -407,7 +432,7 @@ na_xml_writer_get_output_fname( const NAAction *action, const gchar *folder, gin
  * The returned string should be g_free() by the caller.
  */
 gchar *
-na_xml_writer_get_xml_buffer( const NAAction *action, gint format )
+na_xml_writer_get_xml_buffer( const NAObjectAction *action, gint format )
 {
 	gchar *uuid;
 	NAXMLWriter *writer;
@@ -416,23 +441,23 @@ na_xml_writer_get_xml_buffer( const NAAction *action, gint format )
 	int textlen;
 	gchar *buffer;
 
-	g_return_val_if_fail( action, NULL );
+	g_return_val_if_fail( action || format == IPREFS_EXPORT_FORMAT_GCONF_SCHEMA, NULL );
 
-	uuid = na_action_get_uuid( action );
+	uuid = action ? na_object_get_id( action ) : NULL;
 	writer = xml_writer_new( uuid );
 	g_free( uuid );
 
 	switch( format ){
-		case FORMAT_GCONFSCHEMAFILE_V1:
-		case FORMAT_GCONFSCHEMAFILE_V2:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V2:
 			doc = create_xml_schema( writer, format, action );
 			break;
 
-		case FORMAT_GCONFENTRY:
+		case IPREFS_EXPORT_FORMAT_GCONF_ENTRY:
 			doc = create_xml_dump( writer, format, action );
 			break;
 
-		case FORMAT_GCONFSCHEMA:
+		case IPREFS_EXPORT_FORMAT_GCONF_SCHEMA:
 			doc = create_gconf_schema( writer );
 			break;
 	}
@@ -452,9 +477,8 @@ na_xml_writer_get_xml_buffer( const NAAction *action, gint format )
 
 /**
  * na_xml_writer_output_xml:
- * @action: the #NAAction to be exported.
- * @filename: the uri of the output filename.
- *  If NULL, the XML buffer is written to stdout.
+ * @action: the #NAObjectAction to be exported.
+ * @filename: the uri of the output filename
  *
  * Exports an action to the given filename.
  */
@@ -466,61 +490,60 @@ na_xml_writer_output_xml( const gchar *xml, const gchar *filename )
 	GFileOutputStream *stream;
 	GError *error = NULL;
 
-	g_debug( "%s: xml=%p (length=%ld), filename=%s",
-			thisfn, ( void * ) xml, g_utf8_strlen( xml, -1 ), filename ? filename : "(null)" );
+	g_return_if_fail( xml );
+	g_return_if_fail( filename && g_utf8_strlen( filename, -1 ));
 
-	if( filename ){
-		file = g_file_new_for_uri( filename );
-
-		stream = g_file_replace( file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error );
-		if( error ){
-			g_warning( "%s: g_file_replace: %s", thisfn, error->message );
-			g_error_free( error );
-			if( stream ){
-				g_object_unref( stream );
-			}
-			g_object_unref( file );
-			return;
-		}
-
-		g_output_stream_write( G_OUTPUT_STREAM( stream ), xml, g_utf8_strlen( xml, -1 ), NULL, &error );
-		if( error ){
-			g_warning( "%s: g_output_stream_write: %s", thisfn, error->message );
-			g_error_free( error );
-			g_object_unref( stream );
-			g_object_unref( file );
-			return;
-		}
-
-		g_output_stream_close( G_OUTPUT_STREAM( stream ), NULL, &error );
-		if( error ){
-			g_warning( "%s: g_output_stream_close: %s", thisfn, error->message );
-			g_error_free( error );
-			g_object_unref( stream );
-			g_object_unref( file );
-			return;
-		}
+	file = g_file_new_for_uri( filename );
 
+	stream = g_file_create( file, G_FILE_CREATE_REPLACE_DESTINATION, NULL, &error );
+	if( error ){
+		g_warning( "%s: %s", thisfn, error->message );
+		g_error_free( error );
 		g_object_unref( stream );
 		g_object_unref( file );
-
-	} else {
-		g_print( "%s\n", xml );
+		return;
 	}
+
+	g_output_stream_write( G_OUTPUT_STREAM( stream ), xml, g_utf8_strlen( xml, -1 ), NULL, &error );
+	if( error ){
+		g_warning( "%s: %s", thisfn, error->message );
+		g_error_free( error );
+		g_object_unref( stream );
+		g_object_unref( file );
+		return;
+	}
+
+	g_output_stream_close( G_OUTPUT_STREAM( stream ), NULL, &error );
+	if( error ){
+		g_warning( "%s: %s", thisfn, error->message );
+		g_error_free( error );
+		g_object_unref( stream );
+		g_object_unref( file );
+		return;
+	}
+
+	g_object_unref( stream );
+	g_object_unref( file );
 }
 
 static xmlDocPtr
-create_xml_schema( NAXMLWriter *writer, gint format, const NAAction *action )
+create_xml_schema( NAXMLWriter *writer, gint format, const NAObjectAction *action )
 {
 	xmlDocPtr doc;
 	xmlNodePtr root_node, list_node;
 	gchar *version, *label, *tooltip, *icon, *text;
 	gboolean enabled;
-	GSList *profiles, *ip;
-	NAActionProfile *profile;
+	GList *profiles, *ip;
+	NAObjectProfile *profile;
 	gchar *profile_dir, *profile_label, *path, *parameters;
 	GSList *basenames, *mimetypes, *schemes;
 	gboolean match, isfile, isdir, multiple;
+	gchar *type;
+	GSList *items;
+	gboolean target_selection, target_background, target_toolbar;
+	gboolean toolbar_same_label;
+	gchar *toolbar_label;
+	GSList *folders;
 
 	doc = xmlNewDoc( BAD_CAST( "1.0" ));
 	root_node = xmlNewNode( NULL, BAD_CAST( NACT_GCONF_SCHEMA_ROOT ));
@@ -528,98 +551,146 @@ create_xml_schema( NAXMLWriter *writer, gint format, const NAAction *action )
 	list_node = xmlNewChild( root_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_LIST ), NULL );
 
 	/* version */
-	version = na_action_get_version( action );
+	version = na_object_action_get_version( action );
 	create_schema_entry( writer, format, NULL, ACTION_VERSION_ENTRY, version, doc, list_node, "string", FALSE, ACTION_VERSION_DESC_SHORT, ACTION_VERSION_DESC_LONG );
 	g_free( version );
 
 	/* label */
-	label = na_action_get_label( action );
-	create_schema_entry( writer, format, NULL, ACTION_LABEL_ENTRY, label, doc, list_node, "string", TRUE, ACTION_LABEL_DESC_SHORT, ACTION_LABEL_DESC_LONG );
+	label = na_object_get_label( action );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_LABEL_ENTRY, label, doc, list_node, "string", TRUE, ACTION_LABEL_DESC_SHORT, ACTION_LABEL_DESC_LONG );
 	g_free( label );
 
 	/* tooltip */
-	tooltip = na_action_get_tooltip( action );
-	create_schema_entry( writer, format, NULL, ACTION_TOOLTIP_ENTRY, tooltip, doc, list_node, "string", TRUE, ACTION_TOOLTIP_DESC_SHORT, ACTION_TOOLTIP_DESC_LONG );
+	tooltip = na_object_get_tooltip( action );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TOOLTIP_ENTRY, tooltip, doc, list_node, "string", TRUE, ACTION_TOOLTIP_DESC_SHORT, ACTION_TOOLTIP_DESC_LONG );
 	g_free( tooltip );
 
 	/* icon name */
-	icon = na_action_get_icon( action );
-	create_schema_entry( writer, format, NULL, ACTION_ICON_ENTRY, icon, doc, list_node, "string", FALSE, ACTION_ICON_DESC_SHORT, ACTION_ICON_DESC_LONG );
+	icon = na_object_get_icon( action );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_ICON_ENTRY, icon, doc, list_node, "string", FALSE, ACTION_ICON_DESC_SHORT, ACTION_ICON_DESC_LONG );
 	g_free( icon );
 
 	/* enabled */
-	enabled = na_action_is_enabled( action );
+	enabled = na_object_is_enabled( NA_OBJECT_ITEM( action ));
 	text = na_utils_boolean_to_schema( enabled );
-	create_schema_entry( writer, format, NULL, ACTION_ENABLED_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_ENABLED_DESC_SHORT, ACTION_ENABLED_DESC_LONG );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_ENABLED_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_ENABLED_DESC_SHORT, ACTION_ENABLED_DESC_LONG );
 	g_free( text );
 
-	profiles = na_action_get_profiles( action );
+	/* items */
+	items = na_object_item_rebuild_items_list( NA_OBJECT_ITEM( action ));
+	text = na_utils_gslist_to_schema( items );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_LIST_ENTRY, text, doc, list_node, "list", FALSE, ACTION_ITEMS_DESC_SHORT, ACTION_ITEMS_DESC_LONG );
+	g_free( text );
+	na_utils_free_string_list( items );
+
+	/* type */
+	type = NA_IS_OBJECT_ACTION( action ) ? g_strdup( "Action" ) : g_strdup( "Menu" );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TYPE_ENTRY, type, doc, list_node, "string", FALSE, ACTION_TYPE_DESC_SHORT, ACTION_TYPE_DESC_LONG );
+	g_free( type );
+
+	/* target selection */
+	target_selection = na_object_action_is_target_selection( action );
+	text = na_utils_boolean_to_schema( target_selection );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TARGET_SELECTION_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_TARGET_SELECTION_DESC_SHORT, ACTION_TARGET_SELECTION_DESC_LONG );
+	g_free( text );
+
+	/* target background */
+	target_background = na_object_action_is_target_background( action );
+	text = na_utils_boolean_to_schema( target_background );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TARGET_BACKGROUND_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_TARGET_BACKGROUND_DESC_SHORT, ACTION_TARGET_BACKGROUND_DESC_LONG );
+	g_free( text );
+
+	/* target toolbar */
+	target_toolbar = na_object_action_is_target_toolbar( action );
+	text = na_utils_boolean_to_schema( target_toolbar );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TARGET_TOOLBAR_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_TARGET_TOOLBAR_DESC_SHORT, ACTION_TARGET_TOOLBAR_DESC_LONG );
+	g_free( text );
+
+	/* toolbar same label */
+	toolbar_same_label = na_object_action_toolbar_use_same_label( action );
+	text = na_utils_boolean_to_schema( toolbar_same_label );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TOOLBAR_SAME_LABEL_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_TOOLBAR_SAME_LABEL_DESC_SHORT, ACTION_TOOLBAR_SAME_LABEL_DESC_LONG );
+	g_free( text );
+
+	/* toolbar label */
+	toolbar_label = na_object_action_toolbar_get_label( action );
+	create_schema_entry( writer, format, NULL, OBJECT_ITEM_TOOLBAR_LABEL_ENTRY, toolbar_label, doc, list_node, "string", TRUE, ACTION_TOOLBAR_LABEL_DESC_SHORT, ACTION_TOOLBAR_LABEL_DESC_LONG );
+	g_free( toolbar_label );
+
+	profiles = na_object_get_items_list( action );
 
 	for( ip = profiles ; ip ; ip = ip->next ){
 
-		profile = NA_ACTION_PROFILE( ip->data );
-		profile_dir = na_action_profile_get_name( profile );
+		profile = NA_OBJECT_PROFILE( ip->data );
+		profile_dir = na_object_get_id( profile );
 
 		/* profile label */
-		profile_label = na_action_profile_get_label( profile );
+		profile_label = na_object_get_label( profile );
 		create_schema_entry( writer, format, profile_dir, ACTION_PROFILE_LABEL_ENTRY, profile_label, doc, list_node, "string", TRUE, ACTION_PROFILE_NAME_DESC_SHORT, ACTION_PROFILE_NAME_DESC_LONG );
 		g_free( profile_label );
 
 		/* path */
-		path = na_action_profile_get_path( profile );
+		path = na_object_profile_get_path( profile );
 		create_schema_entry( writer, format, profile_dir, ACTION_PATH_ENTRY, path, doc, list_node, "string", FALSE, ACTION_PATH_DESC_SHORT, ACTION_PATH_DESC_LONG );
 		g_free( path );
 
 		/* parameters */
-		parameters = na_action_profile_get_parameters( profile );
+		parameters = na_object_profile_get_parameters( profile );
 		create_schema_entry( writer, format, profile_dir, ACTION_PARAMETERS_ENTRY, parameters, doc, list_node, "string", FALSE, ACTION_PARAMETERS_DESC_SHORT, ACTION_PARAMETERS_DESC_LONG );
 		g_free( parameters );
 
 		/* basenames */
-		basenames = na_action_profile_get_basenames( profile );
+		basenames = na_object_profile_get_basenames( profile );
 		text = na_utils_gslist_to_schema( basenames );
 		create_schema_entry( writer, format, profile_dir, ACTION_BASENAMES_ENTRY, text, doc, list_node, "list", FALSE, ACTION_BASENAMES_DESC_SHORT, ACTION_BASENAMES_DESC_LONG );
 		g_free( text );
 		na_utils_free_string_list( basenames );
 
 		/* match_case */
-		match = na_action_profile_get_matchcase( profile );
+		match = na_object_profile_get_matchcase( profile );
 		text = na_utils_boolean_to_schema( match );
 		create_schema_entry( writer, format, profile_dir, ACTION_MATCHCASE_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_MATCHCASE_DESC_SHORT, ACTION_MATCHCASE_DESC_LONG );
 		g_free( text );
 
 		/* mimetypes */
-		mimetypes = na_action_profile_get_mimetypes( profile );
+		mimetypes = na_object_profile_get_mimetypes( profile );
 		text = na_utils_gslist_to_schema( mimetypes );
 		create_schema_entry( writer, format, profile_dir, ACTION_MIMETYPES_ENTRY, text, doc, list_node, "list", FALSE, ACTION_MIMETYPES_DESC_SHORT, ACTION_MIMETYPES_DESC_LONG );
 		g_free( text );
 		na_utils_free_string_list( mimetypes );
 
 		/* is_file */
-		isfile = na_action_profile_get_is_file( profile );
+		isfile = na_object_profile_get_is_file( profile );
 		text = na_utils_boolean_to_schema( isfile );
 		create_schema_entry( writer, format, profile_dir, ACTION_ISFILE_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_ISFILE_DESC_SHORT, ACTION_ISFILE_DESC_LONG );
 		g_free( text );
 
 		/* is_dir */
-		isdir = na_action_profile_get_is_dir( profile );
+		isdir = na_object_profile_get_is_dir( profile );
 		text = na_utils_boolean_to_schema( isdir );
 		create_schema_entry( writer, format, profile_dir, ACTION_ISDIR_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_ISDIR_DESC_SHORT, ACTION_ISDIR_DESC_LONG );
 		g_free( text );
 
 		/* accept-multiple-files */
-		multiple = na_action_profile_get_multiple( profile );
+		multiple = na_object_profile_get_multiple( profile );
 		text = na_utils_boolean_to_schema( multiple );
 		create_schema_entry( writer, format, profile_dir, ACTION_MULTIPLE_ENTRY, text, doc, list_node, "bool", FALSE, ACTION_MULTIPLE_DESC_SHORT, ACTION_MULTIPLE_DESC_LONG );
 		g_free( text );
 
 		/* schemes */
-		schemes = na_action_profile_get_schemes( profile );
+		schemes = na_object_profile_get_schemes( profile );
 		text = na_utils_gslist_to_schema( schemes );
 		create_schema_entry( writer, format, profile_dir, ACTION_SCHEMES_ENTRY, text, doc, list_node, "list", FALSE, ACTION_SCHEMES_DESC_SHORT, ACTION_SCHEMES_DESC_LONG );
 		g_free( text );
 		na_utils_free_string_list( schemes );
 
+		/* folders */
+		folders = na_object_profile_get_folders( profile );
+		text = na_utils_gslist_to_schema( folders );
+		create_schema_entry( writer, format, profile_dir, ACTION_FOLDERS_ENTRY, text, doc, list_node, "list", FALSE, ACTION_FOLDERS_DESC_SHORT, ACTION_FOLDERS_DESC_LONG );
+		g_free( text );
+		na_utils_free_string_list( folders );
+
 		g_free( profile_dir );
 	}
 
@@ -646,7 +717,7 @@ create_schema_entry( NAXMLWriter *writer,
 
 	schema_node = xmlNewChild( list_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_ENTRY ), NULL );
 
-	content = BAD_CAST( g_build_path( "/", NA_GCONF_SCHEMA_PREFIX, path, NULL ));
+	content = BAD_CAST( g_build_path( "/", NAUTILUS_ACTIONS_GCONF_SCHEMASDIR, path, NULL ));
 	xmlNewChild( schema_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_KEY ), content );
 	xmlFree( content );
 
@@ -676,7 +747,7 @@ create_schema_entry( NAXMLWriter *writer,
 	/* fill up the historical format if asked for
 	 * add owner and short and long descriptions
 	 */
-	if( format == FORMAT_GCONFSCHEMAFILE_V1 ){
+	if( format == IPREFS_EXPORT_FORMAT_GCONF_SCHEMA_V1 ){
 		xmlNewChild( schema_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_OWNER ), BAD_CAST( PACKAGE_TARNAME ));
 
 		xmlNewChild( locale_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_SHORT ), BAD_CAST( short_desc ));
@@ -688,19 +759,25 @@ create_schema_entry( NAXMLWriter *writer,
 }
 
 static xmlDocPtr
-create_xml_dump( NAXMLWriter *writer, gint format, const NAAction *action )
+create_xml_dump( NAXMLWriter *writer, gint format, const NAObjectAction *action )
 {
 	xmlDocPtr doc;
 	xmlNodePtr root_node, list_node;
 	gchar *path;
 	gchar *version, *label, *tooltip, *icon, *text;
 	gboolean enabled;
-	GSList *profiles, *ip;
-	NAActionProfile *profile;
+	GList *profiles, *ip;
+	NAObjectProfile *profile;
 	gchar *profile_dir;
 	gchar *profile_label, *parameters;
 	GSList *basenames, *mimetypes, *schemes;
 	gboolean match, isfile, isdir, multiple;
+	gchar *type;
+	GSList *items;
+	gboolean target_selection, target_background, target_toolbar;
+	gboolean toolbar_same_label;
+	gchar *toolbar_label;
+	GSList *folders;
 
 	doc = xmlNewDoc( BAD_CAST( "1.0" ));
 	root_node = xmlNewNode( NULL, BAD_CAST( NACT_GCONF_DUMP_ROOT ));
@@ -712,98 +789,146 @@ create_xml_dump( NAXMLWriter *writer, gint format, const NAAction *action )
 	g_free( path );
 
 	/* version */
-	version = na_action_get_version( action );
+	version = na_object_action_get_version( action );
 	create_dump_entry( writer, format, NULL, ACTION_VERSION_ENTRY, version, doc, list_node, "string" );
 	g_free( version );
 
 	/* label */
-	label = na_action_get_label( action );
-	create_dump_entry( writer, format, NULL, ACTION_LABEL_ENTRY, label, doc, list_node, "string" );
+	label = na_object_get_label( action );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_LABEL_ENTRY, label, doc, list_node, "string" );
 	g_free( label );
 
 	/* tooltip */
-	tooltip = na_action_get_tooltip( action );
-	create_dump_entry( writer, format, NULL, ACTION_TOOLTIP_ENTRY, tooltip, doc, list_node, "string" );
+	tooltip = na_object_get_tooltip( action );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TOOLTIP_ENTRY, tooltip, doc, list_node, "string" );
 	g_free( tooltip );
 
 	/* icon name */
-	icon = na_action_get_icon( action );
-	create_dump_entry( writer, format, NULL, ACTION_ICON_ENTRY, icon, doc, list_node, "string" );
+	icon = na_object_get_icon( action );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_ICON_ENTRY, icon, doc, list_node, "string" );
 	g_free( icon );
 
 	/* enabled */
-	enabled = na_action_is_enabled( action );
+	enabled = na_object_is_enabled( NA_OBJECT_ITEM( action ));
 	text = na_utils_boolean_to_schema( enabled );
-	create_dump_entry( writer, format, NULL, ACTION_ENABLED_ENTRY, text, doc, list_node, "bool" );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_ENABLED_ENTRY, text, doc, list_node, "bool" );
 	g_free( text );
 
-	profiles = na_action_get_profiles( action );
+	/* items */
+	items = na_object_item_rebuild_items_list( NA_OBJECT_ITEM( action ));
+	text = na_utils_gslist_to_schema( items );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_LIST_ENTRY, text, doc, list_node, "list" );
+	g_free( text );
+	na_utils_free_string_list( items );
+
+	/* type */
+	type = NA_IS_OBJECT_ACTION( action ) ? g_strdup( "Action" ) : g_strdup( "Menu" );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TYPE_ENTRY, type, doc, list_node, "string" );
+	g_free( type );
+
+	/* target selection */
+	target_selection = na_object_action_is_target_selection( action );
+	text = na_utils_boolean_to_schema( target_selection );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TARGET_SELECTION_ENTRY, text, doc, list_node, "bool" );
+	g_free( text );
+
+	/* target background */
+	target_background = na_object_action_is_target_background( action );
+	text = na_utils_boolean_to_schema( target_background );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TARGET_BACKGROUND_ENTRY, text, doc, list_node, "bool" );
+	g_free( text );
+
+	/* target toolbar */
+	target_toolbar = na_object_action_is_target_toolbar( action );
+	text = na_utils_boolean_to_schema( target_toolbar );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TARGET_TOOLBAR_ENTRY, text, doc, list_node, "bool" );
+	g_free( text );
+
+	/* toolbar same label */
+	toolbar_same_label = na_object_action_toolbar_use_same_label( action );
+	text = na_utils_boolean_to_schema( toolbar_same_label );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TOOLBAR_SAME_LABEL_ENTRY, text, doc, list_node, "bool" );
+	g_free( text );
+
+	/* toolbar label */
+	toolbar_label = na_object_action_toolbar_get_label( action );
+	create_dump_entry( writer, format, NULL, OBJECT_ITEM_TOOLBAR_LABEL_ENTRY, toolbar_label, doc, list_node, "string" );
+	g_free( toolbar_label );
+
+	profiles = na_object_get_items_list( action );
 
 	for( ip = profiles ; ip ; ip = ip->next ){
 
-		profile = NA_ACTION_PROFILE( ip->data );
-		profile_dir = na_action_profile_get_name( profile );
+		profile = NA_OBJECT_PROFILE( ip->data );
+		profile_dir = na_object_get_id( profile );
 
 		/* profile label */
-		profile_label = na_action_profile_get_label( profile );
+		profile_label = na_object_get_label( profile );
 		create_dump_entry( writer, format, profile_dir, ACTION_PROFILE_LABEL_ENTRY, profile_label, doc, list_node, "string" );
 		g_free( profile_label );
 
 		/* path */
-		path = na_action_profile_get_path( profile );
+		path = na_object_profile_get_path( profile );
 		create_dump_entry( writer, format, profile_dir, ACTION_PATH_ENTRY, path, doc, list_node, "string" );
 		g_free( path );
 
 		/* parameters */
-		parameters = na_action_profile_get_parameters( profile );
+		parameters = na_object_profile_get_parameters( profile );
 		create_dump_entry( writer, format, profile_dir, ACTION_PARAMETERS_ENTRY, parameters, doc, list_node, "string" );
 		g_free( parameters );
 
 		/* basenames */
-		basenames = na_action_profile_get_basenames( profile );
+		basenames = na_object_profile_get_basenames( profile );
 		text = na_utils_gslist_to_schema( basenames );
 		create_dump_entry( writer, format, profile_dir, ACTION_BASENAMES_ENTRY, text, doc, list_node, "list" );
 		g_free( text );
 		na_utils_free_string_list( basenames );
 
 		/* match_case */
-		match = na_action_profile_get_matchcase( profile );
+		match = na_object_profile_get_matchcase( profile );
 		text = na_utils_boolean_to_schema( match );
 		create_dump_entry( writer, format, profile_dir, ACTION_MATCHCASE_ENTRY, text, doc, list_node, "bool" );
 		g_free( text );
 
 		/* mimetypes */
-		mimetypes = na_action_profile_get_mimetypes( profile );
+		mimetypes = na_object_profile_get_mimetypes( profile );
 		text = na_utils_gslist_to_schema( mimetypes );
 		create_dump_entry( writer, format, profile_dir, ACTION_MIMETYPES_ENTRY, text, doc, list_node, "list" );
 		g_free( text );
 		na_utils_free_string_list( mimetypes );
 
 		/* is_file */
-		isfile = na_action_profile_get_is_file( profile );
+		isfile = na_object_profile_get_is_file( profile );
 		text = na_utils_boolean_to_schema( isfile );
 		create_dump_entry( writer, format, profile_dir, ACTION_ISFILE_ENTRY, text, doc, list_node, "bool" );
 		g_free( text );
 
 		/* is_dir */
-		isdir = na_action_profile_get_is_dir( profile );
+		isdir = na_object_profile_get_is_dir( profile );
 		text = na_utils_boolean_to_schema( isdir );
 		create_dump_entry( writer, format, profile_dir, ACTION_ISDIR_ENTRY, text, doc, list_node, "bool" );
 		g_free( text );
 
 		/* accept-multiple-files */
-		multiple = na_action_profile_get_multiple( profile );
+		multiple = na_object_profile_get_multiple( profile );
 		text = na_utils_boolean_to_schema( multiple );
 		create_dump_entry( writer, format, profile_dir, ACTION_MULTIPLE_ENTRY, text, doc, list_node, "bool" );
 		g_free( text );
 
 		/* schemes */
-		schemes = na_action_profile_get_schemes( profile );
+		schemes = na_object_profile_get_schemes( profile );
 		text = na_utils_gslist_to_schema( schemes );
 		create_dump_entry( writer, format, profile_dir, ACTION_SCHEMES_ENTRY, text, doc, list_node, "list" );
 		g_free( text );
 		na_utils_free_string_list( schemes );
 
+		/* folders */
+		folders = na_object_profile_get_folders( profile );
+		text = na_utils_gslist_to_schema( folders );
+		create_dump_entry( writer, format, profile_dir, ACTION_FOLDERS_ENTRY, text, doc, list_node, "list" );
+		g_free( text );
+		na_utils_free_string_list( folders );
+
 		g_free( profile_dir );
 	}
 
@@ -864,11 +989,18 @@ create_gconf_schema( NAXMLWriter *writer )
 	list_node = xmlNewChild( root_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_LIST ), NULL );
 
 	create_gconf_schema_entry( writer, ACTION_VERSION_ENTRY       , doc, list_node, "string", ACTION_VERSION_DESC_SHORT     , ACTION_VERSION_DESC_LONG     , NAUTILUS_ACTIONS_CONFIG_VERSION, FALSE );
-	create_gconf_schema_entry( writer, ACTION_LABEL_ENTRY         , doc, list_node, "string", ACTION_LABEL_DESC_SHORT       , ACTION_LABEL_DESC_LONG       , "", TRUE );
-	create_gconf_schema_entry( writer, ACTION_TOOLTIP_ENTRY       , doc, list_node, "string", ACTION_TOOLTIP_DESC_SHORT     , ACTION_TOOLTIP_DESC_LONG     , "", TRUE );
-	create_gconf_schema_entry( writer, ACTION_ICON_ENTRY          , doc, list_node, "string", ACTION_ICON_DESC_SHORT        , ACTION_ICON_DESC_LONG        , "", FALSE );
-	create_gconf_schema_entry( writer, ACTION_ENABLED_ENTRY       , doc, list_node, "bool"  , ACTION_ENABLED_DESC_SHORT     , ACTION_ENABLED_DESC_LONG     , "true", FALSE );
-	create_gconf_schema_entry( writer, ACTION_PROFILE_LABEL_ENTRY , doc, list_node, "string", ACTION_PROFILE_NAME_DESC_SHORT, ACTION_PROFILE_NAME_DESC_LONG, NA_ACTION_PROFILE_DEFAULT_LABEL, TRUE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_LABEL_ENTRY    , doc, list_node, "string", ACTION_LABEL_DESC_SHORT       , ACTION_LABEL_DESC_LONG       , "", TRUE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TOOLTIP_ENTRY  , doc, list_node, "string", ACTION_TOOLTIP_DESC_SHORT     , ACTION_TOOLTIP_DESC_LONG     , "", TRUE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_ICON_ENTRY     , doc, list_node, "string", ACTION_ICON_DESC_SHORT        , ACTION_ICON_DESC_LONG        , "", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_ENABLED_ENTRY  , doc, list_node, "bool"  , ACTION_ENABLED_DESC_SHORT     , ACTION_ENABLED_DESC_LONG     , "true", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_LIST_ENTRY     , doc, list_node, "list"  , ACTION_ITEMS_DESC_SHORT       , ACTION_ITEMS_DESC_LONG       , "[]", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TYPE_ENTRY     , doc, list_node, "string", ACTION_TYPE_DESC_SHORT        , ACTION_TYPE_DESC_LONG        , "Action", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TARGET_SELECTION_ENTRY  , doc, list_node,   "bool", ACTION_TARGET_SELECTION_DESC_SHORT  , ACTION_TARGET_SELECTION_DESC_LONG  ,  "true", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TARGET_BACKGROUND_ENTRY , doc, list_node,   "bool", ACTION_TARGET_BACKGROUND_DESC_SHORT , ACTION_TARGET_BACKGROUND_DESC_LONG , "false", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TARGET_TOOLBAR_ENTRY    , doc, list_node,   "bool", ACTION_TARGET_TOOLBAR_DESC_SHORT    , ACTION_TARGET_TOOLBAR_DESC_LONG    , "false", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TOOLBAR_SAME_LABEL_ENTRY, doc, list_node,   "bool", ACTION_TOOLBAR_SAME_LABEL_DESC_SHORT, ACTION_TOOLBAR_SAME_LABEL_DESC_LONG,  "true", FALSE );
+	create_gconf_schema_entry( writer, OBJECT_ITEM_TOOLBAR_LABEL_ENTRY     , doc, list_node, "string", ACTION_TOOLBAR_LABEL_DESC_SHORT     , ACTION_TOOLBAR_LABEL_DESC_LONG     , "", TRUE );
+	create_gconf_schema_entry( writer, ACTION_PROFILE_LABEL_ENTRY , doc, list_node, "string", ACTION_PROFILE_NAME_DESC_SHORT, ACTION_PROFILE_NAME_DESC_LONG, NA_OBJECT_PROFILE_DEFAULT_LABEL, TRUE );
 	create_gconf_schema_entry( writer, ACTION_PATH_ENTRY          , doc, list_node, "string", ACTION_PATH_DESC_SHORT        , ACTION_PATH_DESC_LONG        , "", FALSE );
 	create_gconf_schema_entry( writer, ACTION_PARAMETERS_ENTRY    , doc, list_node, "string", ACTION_PARAMETERS_DESC_SHORT  , ACTION_PARAMETERS_DESC_LONG  , "", FALSE );
 	create_gconf_schema_entry( writer, ACTION_BASENAMES_ENTRY     , doc, list_node, "list"  , ACTION_BASENAMES_DESC_SHORT   , ACTION_BASENAMES_DESC_LONG   , "[*]", FALSE );
@@ -878,6 +1010,7 @@ create_gconf_schema( NAXMLWriter *writer )
 	create_gconf_schema_entry( writer, ACTION_ISDIR_ENTRY         , doc, list_node, "bool"  , ACTION_ISDIR_DESC_SHORT       , ACTION_ISDIR_DESC_LONG       , "false", FALSE );
 	create_gconf_schema_entry( writer, ACTION_MULTIPLE_ENTRY      , doc, list_node, "bool"  , ACTION_MULTIPLE_DESC_SHORT    , ACTION_MULTIPLE_DESC_LONG    , "false", FALSE );
 	create_gconf_schema_entry( writer, ACTION_SCHEMES_ENTRY       , doc, list_node, "list"  , ACTION_SCHEMES_DESC_SHORT     , ACTION_SCHEMES_DESC_LONG     , "[file]", FALSE );
+	create_gconf_schema_entry( writer, ACTION_FOLDERS_ENTRY       , doc, list_node, "list"  , ACTION_FOLDERS_DESC_SHORT     , ACTION_FOLDERS_DESC_LONG     , "[]", FALSE );
 
 	return( doc );
 }
@@ -895,7 +1028,7 @@ create_gconf_schema_entry( NAXMLWriter *writer,
 
 	schema_node = xmlNewChild( list_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_ENTRY ), NULL );
 
-	content = BAD_CAST( g_build_path( "/", NA_GCONF_SCHEMA_PREFIX, NA_GCONF_CONFIG_PATH, entry, NULL ));
+	content = BAD_CAST( g_build_path( "/", NAUTILUS_ACTIONS_GCONF_SCHEMASDIR, NA_GCONF_CONFIG_PATH, entry, NULL ));
 	xmlNewChild( schema_node, NULL, BAD_CAST( NACT_GCONF_SCHEMA_KEY ), content );
 	xmlFree( content );
 
